<?php


/**
 * Created by PhpStorm.
 * User: nkd
 * Date: 2016/01/05
 * Time: 16:40
 */
class api {

	/**
	 * @var null エンドポイントURL
	 */
	private $endPoint = null;
	/**
	 * @var null|string クライアントID
	 */
	private $clientId = null;
	/**
	 * @var null|string クライアントシークレット
	 */
	private $clientSecret = null;
	/**
	 * @var null アクセストークン
	 */
	private $accessToken = null;
	/**
	 * @var null 有効期限（秒）
	 */
	private $expiresIn = null;
	/**
	 * @var null apiのエラー
	 */
	public $error = null;
	/**
	 * @var null apiの結果
	 */
	public $result = null;

	/**
	 * api constructor.
	 *
	 * @param string $endPoint エンドポイントURL
	 * @param string $clientId クライアントID
	 * @param string $clientSecret クライアントシークレット
	 */
	public function __construct( $endPoint, $clientId, $clientSecret ) {
		$this->endPoint     = $endPoint;
		$this->clientId     = $clientId;
		$this->clientSecret = $clientSecret;
	}

	/**
	 * curlでneoのapiにアクセスする。
	 * 成功すれば、resultに結果が入る。失敗すればerrorにエラーが入る。
	 *
	 * @param string $url 接続先URL
	 * @param array $params パラメータ
	 *
	 * @return bool
	 */
	private function _curl( $url, $params ) {


		$curl = curl_init( $url );
		curl_setopt( $curl, CURLOPT_POST, true );
		curl_setopt( $curl, CURLOPT_POSTFIELDS, http_build_query( $params ) );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $curl, CURLOPT_FOLLOWLOCATION, true );
		curl_setopt( $curl, CURLOPT_HEADER, false );
		$output = curl_exec( $curl );

		//print_r($output);
		

		$errorno = curl_errno( $curl);//修正　$outputではなく$curl by kamita
		if ( $errorno ) {
			$this->error = array(
				'error'             => 'curl error',
				'error_description' => $errorno . ': ' . curl_error( $curl )
			);

			return false;
		}

		$header = curl_getinfo( $curl );
		curl_close( $curl );//修正　$outputではなく$curl by kamita
		$output = json_decode( $output );

		if ( $header['http_code'] == '200' ) {
			$this->result = $output;

			return true;
		} else {
			$this->error = array(
				'http_code'         => $header['http_code'],
				'error'             => $output->error,
				'error_description' => $output->error_description,
			);

			return false;

		}
	}

	/**
	 * アクセストークンを取得する。
	 *
	 * @return bool
	 */
	public function getToken() {
		if ( empty( $this->accessToken ) || empty( $this->expiresIn ) || $this->expiresIn < time() ) {
			$params = array(
				'path'          => 'token',
				'client_id'     => $this->clientId,
				'client_secret' => $this->clientSecret
			);

			if ( ! $this->_curl( $this->endPoint, $params ) ) {
				return false;
			}

			$this->accessToken = $this->result->access_token;
			$this->expiresIn   = time() + $this->result->expires_in;
		}

		return true;
	}

	/**
	 * アクセストークンを破棄する。
	 *
	 * @return bool
	 */
	public function revokeToken() {
		if ( empty( $this->accessToken ) || empty( $this->expiresIn ) || $this->expiresIn < time() ) {
			return true;
		}

		$params = array(
			'path'          => 'revokeToken',
			'access_token' => $this->accessToken,
		);

		if ( ! $this->_curl( $this->endPoint, $params ) ) {
			return false;
		}

		return true;
	}

	/**
	 * 保存していたアクセストークンと期限を設定する。
	 * トークンに問題なければ、空の配列が帰る。
	 * トークンの再発行がされたたら、新しいトークンが帰る
	 * トークンの再発行に失敗したらfalseがかえる
	 *
	 * @param null $token
	 * @param null $expires_in
	 *
	 * @return array|bool
	 */
	public function setToken( $token = null, $expires_in = null ) {
		$this->accessToken = $token;
		$this->expiresIn   = $expires_in;
		if ( empty( $this->accessToken ) || empty( $this->expiresIn ) || $this->expiresIn < time() ) {
			if ( ! $this->getToken() ) {
				return false;
			}

			return [ 'token' => $this->accessToken, 'expires_in' => $this->expiresIn ];
		}

		return [];
	}

	/**
	 * 読者データを取得する
	 *
	 * @param int $magazineId 　対象となるメルマガのID
	 * @param string $searchText 検索文字列
	 * @param string $searchType 検索方法
	 *
	 * @return bool|null
	 */
	public function getUser( $magazineId, $searchText, $searchType = 'email' ) {
		if ( ! $this->getToken() ) {
			return false;
		}

		$params = array(
			'path'          => 'user',
			'access_token' => $this->accessToken,
			'magazine_id'  => $magazineId,
			'search_type'  => $searchType,
			'search_text'  => $searchText
		);

		if ( ! $this->_curl( $this->endPoint, $params ) ) {
			return false;
		}

		return $this->result;
	}

	/**
	 * 読者データを登録する。
	 *
	 * @param int $magazineId 対象となるメルマガのID
	 * @param array $array 読者データ
	 *
	 * @return bool
	 */
	public function addUser( $magazineId, $array ) {
		if ( ! $this->getToken() ) {
			return false;
		}

		$params = array(
			'path'          => 'userAdd',
			'access_token' => $this->accessToken,
			'magazine_id'  => $magazineId
		);
		$params = array_merge( $params, $array );

		if ( ! $this->_curl( $this->endPoint, $params ) ) {
			return false;
		}

		return true;
	}

	/**
	 * 読者データを更新する。
	 *
	 * @param int $magazineId 対象となるメルマガのID
	 * @param string $email 更新する読者のメルアド
	 * @param array $array 読者データの配列
	 *
	 * @return bool
	 */
	public function updateUser( $magazineId, $email, $array ) {
		if ( ! $this->getToken() ) {
			return false;
		}

		$params = array(
			'path'          => 'userUpdate',
			'access_token' => $this->accessToken,
			'magazine_id'  => $magazineId,
			'email'        => $email,
		);
		$params = array_merge( $params, $array );

		if ( ! $this->_curl( $this->endPoint, $params ) ) {
			return false;
		}

		return true;
	}

	/**
	 * 読者データを削除する。
	 *
	 * @param int $magazineId 　対象となるメルマガのID
	 * @param string $searchText 検索文字列
	 * @param string $searchType 検索方法
	 *
	 * @return bool
	 */
	public function deleteUser( $magazineId, $searchText, $searchType = 'email' ) {
		if ( ! $this->getToken() ) {
			return false;
		}

		$params = array(
			'path'          => 'userDelete',
			'access_token' => $this->accessToken,
			'magazine_id'  => $magazineId,
			'search_type'  => $searchType,
			'search_text'  => $searchText
		);

		if ( ! $this->_curl( $this->endPoint, $params ) ) {
			return false;
		}

		return true;
	}

	/**
	 * 読者行動データを登録する。
	 *
	 * @param string $email データの対象となる読者のメルアド
	 * @param array $array 行動データの配列
	 *
	 * @return bool
	 */
	public function addAction( $email, $array ) {
		if ( ! $this->getToken() ) {
			return false;
		}

		$params = array(
			'path'          => 'actionAdd',
			'access_token' => $this->accessToken,
			'email'        => $email,
		);
		$params = array_merge( $params, $array );

		if ( ! $this->_curl( $this->endPoint, $params ) ) {
			return false;
		}

		return true;
	}

	/**
	 * 読者行動データを取得する。
	 *
	 * @param string $email データの対象となる読者のメルアド
	 * @param int $limit 取得数
	 * @param int $offset オフセット
	 * @param string $sort ソート
	 *
	 * @return bool|null
	 */
	public function getAction( $email, $limit = 10, $offset = 0, $sort = 'desc' ) {
		if ( ! $this->getToken() ) {
			return false;
		}

		$params = array(
			'path'          => 'action',
			'access_token' => $this->accessToken,
			'email'        => $email,
			'limit'        => $limit,
			'offset'       => $offset,
			'sort'         => $sort,
		);

		if ( ! $this->_curl( $this->endPoint, $params ) ) {
			return false;
		}

		return $this->result;
	}

	/**
	 * 無料メルマガ一覧を取得する
	 *
	 * @return bool|null
	 */
	public function getMagazine() {
		if ( ! $this->getToken() ) {
			return false;
		}

		$params = [
			'path'          => 'magazine',
			'access_token' => $this->accessToken,
		];

		if ( ! $this->_curl( $this->endPoint, $params ) ) {
			return false;
		}

		return $this->result;
	}
}

