<?php
/*
Plugin Name: NEOREADERS
Plugin URI: https://www.neo-vps.com/wp/functions/other/neoplugin
Description: NEOの読者限定公開機能
Author: PRJAPAN,INC
Version: 0.2
Author URI: https://www.neo-vps.com/
*/
//直接アクセス非表示
if (!defined('ABSPATH')) exit;


//クラスの呼び出しとインスタンスの作成
include_once __DIR__ . "/api.class.php";


//=================================================
// 管理画面に「NEOREADERS」を追加登録する
//=================================================
add_action('admin_menu', function () {

	//---------------------------------
	// メインメニュー①
	//---------------------------------	
	add_menu_page(
		'NEOREADERS 設定' // ページのタイトルタグ<title>に表示されるテキスト
		,
		'NEOREADERS'   // 左メニューとして表示されるテキスト
		,
		'manage_options'       // 必要な権限 manage_options は通常 administrator のみに与えられた権限
		,
		'neo_menu'        // 左メニューのスラッグ名 →URLのパラメータに使われる /wp-admin/admin.php?page=toriaezu_menu
		,
		'neo_page_contents' // メニューページを表示する際に実行される関数(サブメニュー①の処理をする時はこの値は空にする)
		,
		'dashicons-admin-users'       // メニューのアイコンを指定 https://developer.wordpress.org/resource/dashicons/#awards
		,
		0                             // メニューが表示される位置のインデックス(0が先頭) 5=投稿,10=メディア,20=固定ページ,25=コメント,60=テーマ,65=プラグイン,70=ユーザー,75=ツール,80=設定
	);
});



//=================================================
// メニュー①ページ内容の表示(APIの設定)
//=================================================
function neo_page_contents()
{

	// Check if the user has the required permission
	if (!current_user_can('manage_options')) {
		wp_die(__('You do not have access to this setting page'));
	}

	// Initialize
	$opt_name_api_url = 'neo_api_url';
	$opt_val_api_url = get_option($opt_name_api_url);
	$opt_name_client_id = 'neo_client_id';
	$opt_val_client_id = get_option($opt_name_client_id);
	$opt_name_client_secret = 'neo_client_secret';
	$opt_val_client_secret = get_option($opt_name_client_secret);
	$message_html = "";

	// Process update
// Process update
if (
	isset($_POST[$opt_name_api_url]) &&
	isset($_POST[$opt_name_client_id]) &&
	isset($_POST[$opt_name_client_secret])
) {

	$opt_val_api_url = sanitize_text_field($_POST[$opt_name_api_url]);
	$opt_val_client_id = sanitize_text_field($_POST[$opt_name_client_id]);
	$opt_val_client_secret = sanitize_text_field($_POST[$opt_name_client_secret]);

	update_option($opt_name_api_url, $opt_val_api_url);
	update_option($opt_name_client_id, $opt_val_client_id);
	update_option($opt_name_client_secret, $opt_val_client_secret);


		$message_html = <<<EOF
			
<div class="notice notice-success is-dismissible">
	<p>
		Settings saved successfully.
	</p>
</div>
			
EOF;
	}

	// Display HTML
	echo $html = <<<EOF

{$message_html}

<div class="wrap">
	
	<h2>NEOREADERS ご利用中VPS-NEOのAPIの情報を設定してください</h2>
	
	<form name="form1" method="post" action="">
	
		<p>ご利用のVPS-NEOのURL(末尾が「/neo/」となります。)</p>
		<p>
			<input type="text" name="{$opt_name_api_url}" value="{$opt_val_api_url}" size="72" placeholder="Enter API URL">
		</p>
	
		<p>クライアントID</p>
		<p>
		<input type="text" name="{$opt_name_client_id}" value="{$opt_val_client_id}" size="72" placeholder="Enter Client ID">
	</p>

		<p>クライアントシークレット</p>
		<p>
			<input type="text" name="{$opt_name_client_secret}" value="{$opt_val_client_secret}" size="72" placeholder="Enter Client Secret">
		</p>
	

	
		<p class="submit">
			<input type="submit" name="Submit" class="button-primary" value="Save Changes">
		</p>
	
	</form>
	</div>
EOF;
}


// Adds the meta box to the edit screens of fixed pages and post pages
//=================================================
// 固定ページと投稿ページの閲覧限定設定
//=================================================
function neo_add_meta_box()
{

	add_meta_box('neo_meta_box', 'VPS-NEO読者限定設定', 'neo_meta_box_callback', array('page', 'post'), 'side', 'high');

	//return true;
}
add_action('add_meta_boxes', 'neo_add_meta_box');

// Outputs the content of the meta box
function neo_meta_box_callback($post)
{
	// Retrieve the saved plugin settings
	$neo_settings = get_post_meta($post->ID, 'neo_settings', true);

	if (!is_array($neo_settings)) {
		$neo_settings = array();
	}
?>
	<p>
		<label for="neo_settings_magazine_number">メルマガのIDを記入<br>(複数は「,」で区切り):</label>
		<input type="text" id="neo_settings_magazine_number" name="neo_settings[magazine_number]" value="<?php echo esc_attr($neo_settings['magazine_number']); ?>">
	</p>
<?php
	// Nonce field to validate the form request
	wp_nonce_field('neo_save_settings', 'neo_nonce');
	//return true;
}

// Saves the plugin settings when the post is saved
add_action('save_post', 'neo_save_settings');


//=================================================
// 閲覧限定mag_numbersをメタテーブルに保存
//=================================================
function neo_save_settings($post_id)
{
	// Check if the form request is valid
	if (!isset($_POST['neo_nonce']) || !wp_verify_nonce($_POST['neo_nonce'], 'neo_save_settings')) {

		return;
	}

	// Validate the magazine number
	$magazine_number = $_POST['neo_settings']['magazine_number'];
	if (!preg_match("/^\d+(,\d+)*$/", $magazine_number)) {
	// Invalid magazine number

	return;
	}

	// Save the plugin settings
	update_post_meta($post_id, 'neo_settings', $_POST['neo_settings']);
	return true;
}




//=================================================
// 閲覧限定
//=================================================

function restrict_content($content)
{
	global $api_flag;

	if ( is_user_logged_in() && ( is_single() || is_page() ) ) {


		// get user email
		$current_user = wp_get_current_user();
		$user_email = $current_user->user_email;


		// get magazine numbers from neo_settings meta data
		//文字列"12,14"のようにして取得

		$neo_settings = get_post_meta(get_the_ID(), 'neo_settings', true);

		if (isset($neo_settings) && isset($neo_settings['magazine_number'])) {
		  $mag_numbers = $neo_settings['magazine_number'];
		} else {
		  $mag_numbers = '';
		}
		
		// check if mag_numbers exists
		if (!empty($mag_numbers)) {

			$api_flag=true;
			
			//neoのapiの取得
			$neourl=get_option('neo_api_url');
			$ept=$neourl.'services.php';
			$cid=get_option('neo_client_id');
			$cst=get_option('neo_client_secret');

			//api初期化	

			$api = new api($ept, $cid, $cst);


			//apiが動くか確認
			if ($neourl != null) {
				
				if ($api->getToken()) {
					//成功
					$api_flag = true;
				} else {
					$api_flag = false;
					return "no token";
			
				}
				
			} else {
				$api_flag = false;
				return "no neourl";
			}
			if(!$api_flag){
				return "api no connect";
			}

			//配列化
			$mag_numbers=explode(",",$mag_numbers);
			// check if user is in the magazine's 読者にいるか
			$check_result = direct_neoapi_check($api, $mag_numbers, $user_email);

			if ($check_result) {
				// show page content
				echo "<h4>会員用のコンテンツです</h4>";
				return get_the_content();
			} else {
				// show restricted message
				$message = "特別会員のみ閲覧可能です。(WordPressのログイン済みですが、それだけでは閲覧できません。)";
				return $message;
			}
		} else {

			// マガジン番号が設定されてないから全公開
			return get_the_content();
		}
	} else {

		//ログインしていない人用

		// get magazine numbers from neo_settings meta data
		$mag_numbers = get_post_meta(get_the_ID(), 'neo_settings', true);

		// check if mag_numbers exists　限定設定ある場合は、非ログイン者は皆非公開
		if (!empty($mag_numbers)) {
			// show restricted message
			$message = "特別会員のみ閲覧可能です。 指定の方法でWordPressに登録またはログインしてください。";
			return $message;
		} else {
			//限定設定されていないの全公開
			// show page content without restrictions
			return get_the_content();
		}
	}

	return $content;
}


add_filter('the_content', 'restrict_content');









//該当の複数のマガジン番号リストの中にユーザーがいるか 直接NEOで調べる
function direct_neoapi_check($api, $mag_numbers = null, $email)
{
	global $api_flag;
	$reader_exist_flag = false;



	if ($api_flag == true) {
		//$mag_numbersが配列てでなく数字だったら
		//echo gettype($mag_numbers)."<br>";
		if (gettype($mag_numbers) == "array") {
			
			foreach ($mag_numbers as $magazineId) {
				//return $magazineId."/".$email;

				//apiから読者データを取得
				$user = $api->getUser($magazineId, $email);

				if (!$user) {
					//失敗した場合はエラーを表示
					$fmes="no user in ".$magazineId;
				} else {
					//必要ならアクセストークンを破棄する。
					$api->revokeToken();
					$reader_exist_flag = true;
					$fmes="Exists user in ".$magazineId;
				}
				//return $fmes;
			}
		} elseif (gettype($mag_numbers) == "string") {
			$magazineId = $mag_numbers;

			//apiから読者データを取得
			$user = $api->getUser($magazineId, $email);
			if (!$user) {
				//失敗した場合はエラーを表示
				//print_r( $api->error );
			} else {
				//必要ならアクセストークンを破棄する。
				$api->revokeToken();
				$reader_exist_flag = true;
			}
		} else {
			wp_die("不測のエラーです。apiから読者の存在確認ができません。");
		}
		if ($reader_exist_flag == "true") {
			return true;
		} else {
			return false;
		}
	} else {
		//api効いてないから、false返します。
		return false;
	}
}

//"success," "error," "warning," "info," 
function notice_create($type, $dismiss, $message) {
	$dismiss_html = $dismiss ? 'is-dismissible' : '';
	$message_html = <<<EOF
	
<div class="notice notice-$type $dismiss_html">
	<p>
		$message
	</p>
</div>
	
EOF;
	echo $message_html;
}


?>